#![allow(dead_code)]

pub mod brain {
    tonic::include_proto!("brain");
}

use crate::data::BrainData;
use brain::brain_cli_service_server::BrainCliService;
use brain::brain_daemon_service_server::BrainDaemonService;
use brain::*;
use std::pin::Pin;
use std::sync::Arc;
use tokio::sync::mpsc;
use tokio_stream::{wrappers::ReceiverStream, Stream};
use tonic::{Request, Response, Status, Streaming};

struct BrainDaemonMock {
    data: Arc<BrainData>,
}
struct BrainCliMock {
    data: Arc<BrainData>,
}

#[tonic::async_trait]
impl BrainDaemonService for BrainDaemonMock {
    async fn register_node(
        &self,
        _req: Request<RegisterNodeRequest>,
    ) -> Result<Response<Empty>, Status> {
        todo!()
    }

    type NewVMUpdatesStream = Pin<Box<dyn Stream<Item = Result<NewVmRequest, Status>> + Send>>;

    async fn new_vm_updates(
        &self,
        _req: Request<Streaming<NewVmConfirmation>>,
    ) -> Result<Response<Self::NewVMUpdatesStream>, Status> {
        todo!()
    }

    type DeletedVMUpdatesStream =
        Pin<Box<dyn Stream<Item = Result<DeletedVmUpdate, Status>> + Send>>;

    async fn deleted_vm_updates(
        &self,
        _req: Request<Empty>,
    ) -> Result<Response<Self::DeletedVMUpdatesStream>, Status> {
        todo!()
    }

    type ListVMContractsStream = Pin<Box<dyn Stream<Item = Result<VmContract, Status>> + Send>>;

    async fn list_vm_contracts(
        &self,
        req: Request<ListVmContractsReq>,
    ) -> Result<Response<Self::ListVMContractsStream>, Status> {
        let req = req.into_inner();
        let contracts = self
            .data
            .find_contracts_by_admin_pubkey(&req.node_pubkey)
            .await;
        let (tx, rx) = mpsc::channel(128);
        tokio::spawn(async move {
            for contract in contracts {
                let _ = tx
                    .send(Ok(contract.into()))
                    .await;
            }
        });

        let output_stream = ReceiverStream::new(rx);
        Ok(Response::new(
            Box::pin(output_stream) as Self::ListVMContractsStream
        ))
    }
}

#[tonic::async_trait]
impl BrainCliService for BrainCliMock {
    async fn create_vm_contract(
        &self,
        _req: Request<NewVmRequest>,
    ) -> Result<Response<NewVmConfirmation>, Status> {
        todo!();
    }

    type ListVMContractsStream = Pin<Box<dyn Stream<Item = Result<VmContract, Status>> + Send>>;
    async fn list_vm_contracts(
        &self,
        req: Request<ListVmContractsReq>,
    ) -> Result<Response<Self::ListVMContractsStream>, Status> {
        let req = req.into_inner();
        let contracts = self
            .data
            .find_contracts_by_admin_pubkey(&req.admin_pubkey)
            .await;
        let (tx, rx) = mpsc::channel(128);
        tokio::spawn(async move {
            for contract in contracts {
                let _ = tx
                    .send(Ok(contract.into()))
                    .await;
            }
        });

        let output_stream = ReceiverStream::new(rx);
        Ok(Response::new(
            Box::pin(output_stream) as Self::ListVMContractsStream
        ))
    }

    type ListNodesStream = Pin<Box<dyn Stream<Item = Result<NodeListResp, Status>> + Send>>;
    async fn list_nodes(
        &self,
        req: Request<NodeFilters>,
    ) -> Result<Response<Self::ListNodesStream>, tonic::Status> {
        let req = req.into_inner();
        let nodes = self.data.find_nodes_by_filters(&req).await;
        let (tx, rx) = mpsc::channel(128);
        tokio::spawn(async move {
            for node in nodes {
                let _ = tx
                    .send(Ok(node.into()))
                    .await;
            }
        });

        let output_stream = ReceiverStream::new(rx);
        Ok(Response::new(
            Box::pin(output_stream) as Self::ListNodesStream
        ))
    }
}
